/*!
    \file ft_test.cc
    \brief Flat tree test facility.
    \author Petr Gladkikh
    $Id: ft_test.cc,v 1.4 2002/10/20 15:19:45 petr Exp $
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
*/

#include "util/flat_tree.h"
#include <iostream>

using namespace std;

int cmpVoid(const void* A, const void* B)
{
  if (*(const char**)(A) < *(const char**)(B))
  {
    return 1;
  }
  if (*(const char**)(A) > *(const char**)(B))
  {
    return -1;
  }
  return 0;
}


int cmpInt(const void* A, const void* B)
{
  if (*(int*)A < *(int*)B)
  {
    return 1;
  }
  if (*(int*)(A) > *(int*)(B))
  {
    return -1;
  }
  return 0;
}

using namespace dm;

template <class T>
void printAll(const FBTree<T>& tree)
{
  typename FBTree<T>::Iterator i(&tree);
  i.goRoot();
  cout << "Tree is " << endl;
  while (i.hasNext())
  {
    cout << *i.getNext() << endl; 
  }
}


void testVoidPtr()
{
  dm::uint size = 100;
  void** newBody = (void **)(new char[FBTree<void*>::getFitSize(size)]);
  FBTree<void*> tree(newBody, size, cmpVoid);   
  for (int i=0; i < 1000; i++)    
    tree.insert((void*)testVoidPtr); 
  printAll<void*>(tree);  
  cout << "going to remove" << endl;
  tree.remove((void*)testVoidPtr);
  printAll<void*>(tree);  
}


void testInt()
{
  dm::uint size = 100;
  int* newBody = (int*)(new char[FBTree<int>::getFitSize(size)]);
  FBTree<int> tree(newBody, size, cmpInt);   
  for (int i=0; i < 100; i++)    
    tree.insert(i); 
  printAll<int>(tree);  
  cout << "going to remove" << endl;
  for (int i=0; i < 100; i++)    
    tree.remove(i);
  printAll<int>(tree);  
}


int main(int argc, char** argv)
{
  cout << " FBTree test." << endl;
  testVoidPtr();
  cout << " ----------- " << endl;
  testInt();
}

