/*!
    \file pm_test.cc
    \brief Pointer mask container
    \author Petr Gladkikh, 2001
    $Id: pm_test.cc,v 1.3 2002/10/20 15:19:45 petr Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
 */

#include <iostream>
#include "util/pointer_mask.h"
#include "debug/tester.h"
#include <unistd.h>
#include <typeinfo>

using namespace std;

#undef NDEBUG 

const dm::uint BIG = 10000;


// --------------------------------------------------------
// Tests


bool test_a_1()
{
    unsigned pool[BIG];
    PointerMask pm;
    pm.init((void**)pool, (void**)(pool + BIG - 1));
    PointerMask::Statistics stat(&pm);
    return stat.pointers_number == 0;   
}


bool test_a_2()
{
    unsigned pool[BIG];
    PointerMask pm;
    void* bottom = pm.init((void**)pool, (void**)(pool + BIG - 1));
    unsigned size = (void**)bottom - (void**)pool;
    assert(pool < bottom);  
    assert(bottom < pool + BIG);  
    for (int ileave = 1; ileave < 100; ileave++)
    {
	for (dm::uint i = 0; i < size; i++)
	{
	    if (i % ileave)
	    {
		pool[i] = -i;
		pm.registerPointer(pool + i);
	    }
	    else
	    {
		pool[i] = i;
		pm.forgetPointer(pool + i);
	    }
	   
	    if (i % ileave)
	    {
		assert(pool[i] == -i);
		assert(pm.isRegistered(pool + i));
	    }
	    else
	    {
		assert(pool[i] == i);
		assert(!pm.isRegistered(pool + i));
	    }
	}
    }
	    
    return true;
}


bool test_a_3()
{
    unsigned pool[BIG];
    PointerMask pm;
    void* bottom = pm.init((void**)pool, (void**)(pool + BIG - 1));
    unsigned size = (void**)bottom - (void**)pool;
    assert(pool < bottom);  
    assert(bottom < pool + BIG);  
    for (int ileave = 1; ileave < 100; ileave++)
    {
	for (dm::uint i = 0; i < size; i++)
	{
	    if (i % ileave)
	    {
		pool[i] = -i;
		pm.registerPointer(pool + i);
	    }
	    else
	    {
		pool[i] = i;
		pm.forgetPointer(pool + i);
	    }
	}
	for (dm::uint i = 0; i < size; i++)
	{
	    if (i % ileave)
	    {
		assert(pool[i] == -i);
		assert(pm.isRegistered(pool + i));
	    }
	    else
	    {
		assert(pool[i] == i);
		assert(!pm.isRegistered(pool + i));
	    }
	}
    }
	    
    return true;
}


// -------------------------------------------------------- 


TestCase tests[] =
{
    { test_a_1, "a1" },
    { test_a_2, "a2" },
    { test_a_3, "a3" },
    { 0,        ""   }  /* nil function pointer is the stop token */
};


// end tests list
// -------------------------------------------------------- 


int main(int argc, char** argv)
{
    cout << "Pointer mask test." << endl;
    if (runTests(tests))
    {
	cout << "TESTS PASSED" << endl;
	return 0;
    }
    else
    {
	cout << endl;
	cout << "TESTS FAILED" << endl;
	return 1;
    };
}

